//==============================================================================
// Javascript for Netburner Support
// DESCRIPTION: This file contains the basic code showing how to write state
// data from the netburner to a javascript object. It also shows how to use
// AJAX to call back to the netburner to update information on the page without
// doing a page refresh. I've also included some examples of how to create a Select drop
// down just from javascript and how to create one based on data returned from the NB
// via an Ajax call. I created three different AJAX calls so you can see one way to
// parse and handle different requests.
// The _machineState object is written by Netburner method WriteSystemVarsToJavaScript()
// You will see a call to that method in the <HEAD> section of index.htm
//
// Coding Standard Note: I used to use a trailing underscore_ for my member variable names
// after years of coding in C# I've gotten in the habit of using a _leading underscore. 
// The javascript and associated C++ code uses a combination of both as I haven't updated
// all the old code. In C++ it's probably not a recommended practice to use leading underscores
// as some of the variable mangling that C++ does uses them also but I've never had a problem and
// I find the advantages outweigh this concern (at least until it bites me in the butt).
// Copyright 2009 Tod Gentille Syncor Systems,Inc.
//See syncor.js for applicable licensing information.
//==============================================================================


//==============================================================================
// Executable code that runs when the page that includes this file loads.
//==============================================================================



if(typeof syn =='undefined') 
	  throw("shell requires the syn Libary from syncor.js");
var reqObject ={
	Version 						: '1.0.0',
	// Constants for Making Ajax Requests
	REQUEST_CMD 		   	: "request",
	MACHINE_STATE			   : "machineState",
	SAMPLE_DATA					: "sampleData",
	DEBUG_OPTIONS 				: "debugOptions"
}

//This code will just run since it isn't inside an object of any sort
//and it was included in index.htm inside a Script element. 
//In effect it runs down to the addEvent call. The rest of the code is then
//invoked from either NB function callbacks, posts, the timer expiring or AJAX calls.
var shell = new Object();
shell._sampleData = {};
shell.TICK_ID = "tick";
shell.RANDOM_ID = "random";
shell.SAMPLE_DROP_DOWN_ID = "sampleDropDownBox";
shell.DEBUG_DROP_DOWN_FROM_AJAX_ID = "sampleAjaxDebugDropDownBox"
syn.AddEvent(window, 'load', doLoad);




//==============================================================================
// When the form loads set up all the GUI elements to reflect the current
// state of the NetBurner, then start the Clock that triggers the  AJAX
// callbacks
//==============================================================================
function doLoad()
{
   shell.setupCheckboxes();
   shell.addSampleSelect();
   shell.setupSampleSelectViaAjax();
   shell.StartClock();
   shell.selectDebug(); //in the form set the index of the NB's current debug state  
}



//==============================================================================
// Loop through the select options for the Debug setting
// and when the text matches the value written
// out to the _machineState object set the selected property to true. This will
// make that option shows in the popup.
//==============================================================================
shell.selectDebug = function()
{
  var debug_list = document.forms[0].cmb_debug;
  var index = _machineState.debugState;

  debug_list.options[index].selected = true;

// If you write out the text of the select box to the machine state
// then you find the matching item as follows
/*
  for (var idx = 0; idx < debug_list.options.length; ++idx)
  {
    if (debug_list.options[idx].text == _machineState.debugState)
    {
        debug_list.options[idx].selected = true;
    }
    else debug_list.options[idx].selected = false;
  }
*/
}


//==============================================================================
// Check the _machineState settings and enable the corresponding checkbox
// if the corresponding _machineState setting is true.
// Checkboxes are not checked by default
//==============================================================================
shell.setupCheckboxes = function()
{
   var the_form = document.forms[0];
   var ms = _machineState;
   if (ms.checkBoxState) the_form.ckb_test.checked = true;
   else the_form.ckb_test.checked  = false;
}


//==============================================================================
// When the timer expires it ends up calling this function.
//==============================================================================
shell.CallbackNetburner = function()
{
	var request = syn.MakeUrlParmList(reqObject.REQUEST_CMD, reqObject.SAMPLE_DATA);
	syn.MakeAjaxRequest(request,shell.CreateSampleData);
}

shell.CreateMachineState = function(resultData)
{
	this._machineState = syn.EvalJsonObject(resultData);
}

shell.CreateSampleData= function(resultData)
{
	_sampleData = syn.EvalJsonObject(resultData);

	 var description = syn.DescribeObject(_sampleData);
	 $('objectSample').innerHTML = description;
	if (this._sampleData != null)
	{ 
	   syn.AddTextToCleanNodeViaId(_sampleData.systemTick, shell.TICK_ID);
      syn.AddTextToCleanNodeViaId(_sampleData.randomValue, shell.RANDOM_ID)
      syn.HideMe("ajaxTest")
   }
}


//==============================================================================
// Set up a timer for the callback to the NetBurner
//==============================================================================
shell.timerTicks_ = 3;
shell.StartClock= function()
{
	--shell.timerTicks_;
	setTimeout("shell.StartClock()",200); // delay is in milliseconds
	if(shell.timerTicks_==0)
	{
		shell.timerTicks_ = 3;
		shell.CallbackNetburner();
	}
}


shell.addSampleSelect = function()
{
	var node_id = shell.SAMPLE_DROP_DOWN_ID;
	var drop_down_list = syn.CreateSelectSequential(10,1);
	var the_node = syn.GetCleanNode(node_id);
	the_node.appendChild(drop_down_list);
}


shell.setupSampleSelectViaAjax = function()
{
	var request = syn.MakeUrlParmList(reqObject.REQUEST_CMD, reqObject.DEBUG_OPTIONS);
	syn.MakeAjaxRequest(request,shell.CreateSampleSelect);
}

shell.CreateSampleSelect= function(resultData)
{
	var debug_response = syn.EvalJsonObject(resultData);
	var debug_select = syn.CreateSelect(debug_response.options);
	debug_select.selectedIndex = debug_response.selectedItem;

	if (debug_select != null)
	{ 
		var node_id = shell.DEBUG_DROP_DOWN_FROM_AJAX_ID;
		var the_node = syn.GetCleanNode(node_id);
		the_node.appendChild(debug_select);
   }
}


//******************************************************************************
// End AJAX Code
//******************************************************************************

