/*
 * HtmlServices.cpp
 *
 *  Created on: Nov 18, 2009
 *      Author: Tod
 */

#include "HtmlServices.h"

#include <iostream>
#include <sstream>
//#include <iosfwd>
#include <utils.h> //has TimeTick
#include <string>
#include <iosys.h> //for writestring
#include <constants.h>
#include <htmlfiles.h>
#include "ShellCommon.h"
#include <http.h>
#include <map>
#include <vector>

#include "Debug.h"

using namespace std;

//==============================================================================
// This page holds the C++ code that is exposed to the HTML pages for use
// as callbacks. We minimize the callbacks by using javascript when possible.
// One of the primary purposes of this code is to write the machine state out
// to the web page using a javascript object with properties. Then the javascript
// can analyze that info to customize the HTML to reflect the machine state.
//==============================================================================
//const vector<string> HtmlServices::AjaxCommands ("request=machinestate", "request=sampleData", "request=debugOptions",3);
//const char* HtmlServices::AjaxCommands[] =
//{
//  "request=machinestate",
//  "request=sampleData",
//  "request=debugOptions"
//};


HtmlServices::AjaxFunctionMap HtmlServices::AjaxCommands;
const string HtmlServices::_requestToken = "request";


bool HtmlServices::_testCheckbox = true;

const string TERM_STRING = "\r\n";

HtmlServices::HtmlServices()
{
	AjaxCommands.insert(make_pair(_requestToken+"=machinestate", &WriteJsonMachineState));
	AjaxCommands.insert(make_pair(_requestToken+"=sampleData", &WriteSampleJsonData));
	AjaxCommands.insert(make_pair(_requestToken+"=debugOptions", &WriteDebugOptionsJsonData));
}


double HtmlServices::GetRandomNumber()
{
	int random_number = rand();
	random_number %= 1000000;
	double value = random_number / 1000.0;

	return value;
}

void HtmlServices::ProcessAjaxFunction(int sock, string url)
{
	string::size_type found_string = url.find(_requestToken);
	if (found_string == string::npos)
	{
		ostringstream msg;
		msg << "Url did not contain " << _requestToken << endl;
		Debug::Write(msg.str());
		return;
	}

	string command = url.substr(found_string);
	AjaxFunctionMap::iterator ajax_iter = AjaxCommands.find(command);
	if (ajax_iter != AjaxCommands.end())
	{
		//cout << "found a command " << ajax_iter->first << endl;
		ajax_iter->second(sock);
	}
	else
	{
		cout << "could not find match for " << command << endl;
	}

}

void HtmlServices::WriteJsonMachineState(int sock)
{
	const string MACHINE_STATE_VAR = "_machineState";
	const string DEBUG_STATE_PROP = "debugState";
	const string CHECKBOX_STATE_PROP = "checkBoxState";

	const string TRUE_STR = "true";
	const string FALSE_STR = "false";
	ostringstream java_script;

	java_script << "var " << MACHINE_STATE_VAR << "=\n{\n";
	java_script << "\t " << DEBUG_STATE_PROP << ": " << Debug::GetLevel() << ",\n";
	java_script << "\t " << CHECKBOX_STATE_PROP << ": " << (_testCheckbox ? TRUE_STR : FALSE_STR) << "\n};";
	writestring(sock, java_script.str().c_str());

	cout << java_script.str().c_str() << endl;
}

//==============================================================================
// Example of how to well a well-formed JSON object for creating a select
// drop down list. Also using map from the C++ std library.
//==============================================================================
void HtmlServices::WriteDebugOptionsJsonData(int sock)
{
	map<int, string> debug_options;
	map<int, string>::iterator map_iter;
	string qt = "\"";

	debug_options.insert(make_pair(1, "The First Item"));
	debug_options.insert(make_pair(2, "Another Item"));
	debug_options.insert(make_pair(3, "The Last Item"));

	ostringstream java_script;
	java_script << "{options:[";
	for (map_iter = debug_options.begin(); map_iter != debug_options.end();)
	{
		java_script << "{value:" << qt << map_iter->first << qt << ",display:" << qt << map_iter->second << qt << "}";
		if (++map_iter != debug_options.end()) java_script << ",";
		else java_script << "]";
	}

	//If your code wants to pre-select a certain item on the select you can add more
	//information to the object and then your js code can use that to select the
	//proper item.
	java_script << ",selectedItem:1}"; //Select the 2nd item (index starts at 0)
	writestring(sock, java_script.str().c_str());
}

//==============================================================================
// Example of how to write out a well formed JSON object that the receiving script
// can just use eval() on to create a variable. Then the script can easily extract
// the values and use them on the page.
// In SupportScripts.js you will see function CreateSampleData(resultData)
// that gets set as the responder to our AJAX call and it processes the data created
// here.
//==============================================================================
void HtmlServices::WriteSampleJsonData(int sock)
{
	const string SYSTEM_TICK = "systemTick";
	const string RANDOM_VALUE = "randomValue";
	ostringstream java_script;

	Debug::Write("J", dl_TACITURN, mt_GREATER_THAN_OR_EQUAL);

	java_script << "{\n";
	java_script << "\t " << SYSTEM_TICK << ": " << TimeTick << ",\n";
	java_script << "\t " << RANDOM_VALUE << ": " << GetRandomNumber() << "\n}";
	writestring(sock, java_script.str().c_str());

}

//==============================================================================
//Example of how you would write out HTML code to a passed reference to an
//output string stream. The caller of this would
//writestring (sock, response.str().c_str())
//This method produces a similar result to the WriteSampleJsonData method that we are
//actually using.
//the data out to the web page. The script on the web page would have a function like
//function UpdateAjaxResponseArea(resultData)
//{
//	document.getElementById('ajaxTest').innerHTML = resultData;
//}
// set as the responder to the AJAX call. That function would find a div with the id
// of "ajaxTest"  and replace all that DIV's html with the html created by this method
// innerHTML works pretty well but is a deprecated tag. It's better to just send back
// JSON objects and let the script insert values into existing portions of already coded
// up HTML rather than trying to code HTML up here. On occasion though I have
// found this technique useful in special case.
//==============================================================================
void HtmlServices::GetSampleHtmlTable(ostringstream& response)
{
	response.precision(3);
	response.setf(ios::fixed, ios::floatfield);
	response.width(7);

	response << "<table border='1' width ='550'>";
	response << "<tr><th>Tick Count</th> <th>Random Number</th>  </tr>";
	response << "<tr><td>" << TimeTick << "</td>";
	response.fill('0');

	response << "<td>";
	response.width(7);
	response << GetRandomNumber() << "</td> </tr>";
	response << " </table>";

}

//==============================================================================
// Post Handler - When the user submits a form this method is called
// Set up using SetNewPostHandler() via the RegisterPost() method
//==============================================================================
int HtmlServices::PostHandler(int sock, char *url, char *pData, char *rxBuffer)
{
	const string REDIRECT_PAGE = "index.htm";
	cout << "In MyDoPost..." << endl;
	//In a Real application one would likely have multiple forms.
	//one would then select the proper processing function by dispatching based
	//on the url data member.
	//In this case we will ignore that....
	const int MAX_CB_RET_LEN = 3; //Checkboxes only exist in returned post if the box is enabled.
	const int MAX_DEBUG_SIZE_LEN = 16; //65536 wold be our biggest value so make it one extra
	//Declare the strings for all the HTML interface widget names
	const string TEST_CB = "ckb_test";
	const string DEBUG_CMB = "cmb_debug";

	//ExtractPostData returns -1 if field not found, 0 if found but empty
	char checkbox_data[MAX_CB_RET_LEN];
	char debug_setting[MAX_DEBUG_SIZE_LEN];

	//Checkboxes only show up in the post if they are true, so just look to see if it is here
	if (ExtractPostData(TEST_CB.c_str(), pData, checkbox_data, sizeof(checkbox_data)) != -1)
	{
		_testCheckbox = true;
	}
	else _testCheckbox = false;

	if (ExtractPostData(DEBUG_CMB.c_str(), pData, debug_setting, sizeof(debug_setting)) != -1)
	{
		const string scpi_debug_prefix = "SYST:DEBUG  ";
		ostringstream scpi_cmd;
		scpi_cmd << scpi_debug_prefix << " " << debug_setting;
		cout << "SCPI command: " << scpi_cmd.str() << endl;
		Debug::SetLevel(debug_setting);
	}
	else
	{
		cout << "Error in extracting text from debug setting popup." << endl;
	}

	//We have to respond to the post with a new HTML page...
	//In this case we will redirect so the browser will go to that URL for the response...
	RedirectResponse(sock, REDIRECT_PAGE.c_str());

	return 0;
}

//==============================================================================
// Install the method we want to have handle form submissions
//==============================================================================
void HtmlServices::RegisterPost() const
{
	SetNewPostHandler(PostHandler);
}

