/*
 * Startup.cpp
 *
 *  Created on: Nov 18, 2009
 *      Author: Tod
 */
#include "predef.h"
#include <startnet.h>
#include <ctype.h>
#include <dhcpclient.h>
#include <autoupdate.h>
#include <smarttrap.h>
#include <NetworkDebug.h>
#include <iostream>
#include "TcpServer.h"
#include "TcpServerReset.h"

#include "Startup.h"
#include "Debug.h"

using namespace std;

//==============================================================================
//The stacks needed by OSCreateStack
//==============================================================================
DWORD TcpServerStack[USER_TASK_STK_SIZE] __attribute__((aligned(4)));
DWORD TcpResetStack[USER_TASK_STK_SIZE] __attribute__((aligned(4)));

//A higher number gives a task a lower priority - 64 levels available total
//but 4 highest and lowest reserved by uCOS II leaving us 56
//MAIN_PRIO defined by Netburner in ./include/constants.h as 50
const int TCP_SERVER_PRIO = MAIN_PRIO + 1;
const int TCP_RESET_PRIO = TCP_SERVER_PRIO + 1;

Startup::Startup(HtmlServices& theHtmlServices, TcpServer& theTcpServer, TcpServerReset& theTcpServerReset) :
	_htmlServices(theHtmlServices), _tcpServer(theTcpServer), _tcpServerReset(theTcpServerReset)
{
	Debug::SetLevel(dl_TACITURN);
}


//==============================================================================
// Standard init calls that need to be done for various classes that are part
// of the NetBurner shell.
//==============================================================================
void Startup::ShellInit(void)
{
	if (EthernetIP == 0) GetDHCPAddress();
	OSChangePrio(MAIN_PRIO);
	EnableAutoUpdate();
	cout << "AutoUpdate enabled, Smart Traps enabled." << endl;

	OSTimeDly(50);
    EnableSmartTraps();
    StartHTTP();
    //NOTE:  RegisterPost() must be called after StartHttp();
    _htmlServices.RegisterPost();
    GetAndShowDhcpAddress();

	// ------------------------------------------------------
	// spin off TCP task thread at one priority level lower
	OSTaskCreate(TcpServer::InitializeWrapper, 0, (void*) &TcpServerStack[USER_TASK_STK_SIZE], (void*) TcpServerStack,
			TCP_SERVER_PRIO);

	// ------------------------------------------------------
	// spin off TCP reset thread at one priority level lower
	OSTaskCreate(TcpServerReset::InitializeWrapper, 0, (void*) &TcpResetStack[USER_TASK_STK_SIZE],
			(void*) TcpResetStack, TCP_RESET_PRIO);

	MainEventLoop();
}

void Startup::GetAndShowDhcpAddress()
{
    GetDHCPAddress();
    cout << "DHCP assigned the IP address of: ";
    ShowIP(EthernetIP);
    cout << endl;
}


void Startup::MainEventLoop()
{
	const bool FOREVER = true;
	const int MAILBOX_WAIT_TICKS = 20;
	OSTimeDly(MAILBOX_WAIT_TICKS);

	OS_MBOX& tcp_mailbox = TcpServer::GetMailbox();

	while (FOREVER)
	{

		BYTE err;
		void * pdata_from_tcp = OSMboxPend( &tcp_mailbox, MAILBOX_WAIT_TICKS, &err);

		if (err != OS_TIMEOUT)
		{
			Debug::Write("!",dl_TACITURN);
			string puser_command = (char*) pdata_from_tcp;
			ParseIncomingMsg(puser_command);
		}
		else
		{
			Debug::Write(".",dl_TACITURN);
		}
	}
}


void Startup::ParseIncomingMsg(string userMsg)
{

	cout << userMsg;
	OS_SEM& msg_processed_sem = TcpServer::GetSemaphore();
	OSSemPost(&msg_processed_sem);
}

