//------------------------------------------------------------------------------
/*
 * Author:: Tod Gentille
 * Date:: Nov 18, 2009
 * Workfile:: tcpServer.h
 * $Revision:: 1
 * (C) Copyright Syncor Systems, Inc.
 */
//------------------------------------------------------------------------------

#ifndef TCPSERVER_H_
#define TCPSERVER_H_

#include <nettypes.h>
#include <ucos.h>
#include "TcpServerReset.h"




class TcpServer
{
	public:
		TcpServer(TcpServerReset* theTcpServerReset);
		//==============================================================================
		//Starts up the TCP server on the  port 23 and listens for a connection
		//once a connection is established it listens for messages and calls ParseReadData()
		//and posts a mailbox message so the Main thread knows we got info.
		//==============================================================================
		static void InitializeWrapper(void* const pd);

		//static void TcpServerReset(void * const pd);
		void ClearWaitFlag();
		static OS_MBOX& GetMailbox();
		static OS_SEM& GetSemaphore();

		//============================================================================
		//Provide a public member to expose the _tcpConnection just so we
		//can send messages back to the client from any module.
		//============================================================================
		int GetTcpConnection()
		{
			return _tcpConnection;
		}



		void SetDebugTcp(bool value) {_debuggingTcp = value;}

	private:
		static const int READ_TIMEOUT_SECS = 5;
		static const int READ_TIMEOUT_TICKS = TICKS_PER_SECOND * READ_TIMEOUT_SECS;
		static const int TCP_LISTEN_PORT = 23;
		static const int RX_BUF_SIZE = 4095;
		static const int MAX_WAIT_MSG_HANDLED_TICKS = TICKS_PER_SECOND * 10;


		int _tcpConnection;
		char _rxBuffer[RX_BUF_SIZE];
		bool _messageWaiting;
		TcpServerReset* _tcpServerReset;
		bool _resetIssued;
		bool _connectionIsOpen;
		bool _debuggingTcp;

		static OS_SEM _msgProcessed;
		static OS_MBOX _mailbox;


		void ProcessMessage(int num_bytes_read);
		void SetConnectionStatus();

		bool ConnectionIsOpen();

		void CloseConnection();
		void ProcessMessagesWhileConnectionOpen(IPADDR client_addr);
		void Initialize(const void * const pd);
		int ReadUntilNoClientAvail(IPADDR clientAddr);
		bool ResetCommandIssued();
};

#endif /* TCPSERVER_H_ */
