/*
 * TcpServerReset.cpp
 *
 *  Created on: Dec 7, 2009
 *      Author: Tod
 */

#include "TcpServerReset.h"
#include "ServiceProvider.h"
#include <nettypes.h>
#include <startnet.h>
#include <tcp.h>

#include <iostream>

using namespace std;

TcpServerReset::TcpServerReset()
{
	ClearConnectionReset();
}


void TcpServerReset::InitializeWrapper(void* const pd)
{
	ServiceProvider::GetTcpServerResetAsSingleton().Initialize(pd);
}

//==============================================================================
// We allow a connection on port 0xd1e 3358, if such a connection is established
// we reset the std tcp connection.  This is provided because if the client that
// connects crashes, the NB doesn't drop the connection. That means when the client
// reboots it can't reconnect to the device unless the power is cycled.
// This process just allows a way for a host to force the NB to drop any current
// connection and then reconnect. Obviously if your LAN is not secure this can
// be abused. Other approaches are to use ping to make sure the connection is up
// but that generates often unwanted network traffic and if the client is a fast
// rebooting embedded device (as opposed to say Windows) then the ping may work
// if the client crashes and reboots within the ping cycle, which would leave you with
// a NB that still thinks it's connected even though it is not.
//==============================================================================
void TcpServerReset::Initialize(void * const pd)
{
	//Set up the listening TCP socket
	int tcp_listen = listen(INADDR_ANY, RESET_PORT, 1); //max pending = 1 ; used to be 5

	if (tcp_listen > 0) //did the listen port setup
	{
		IPADDR client_addr;
		WORD port;

		while (true)
		{
			cout << "tcp:Allowing a tcp reset via connection on port: " << RESET_PORT << "..." << endl;
			//Wait to accept an incoming connection
			int tcp_connection = accept(tcp_listen, &client_addr, &port, 0); //timeout =0 wait forever
			//If anyone connects we are going to reset.
			cout << "tcp: Reset received from ";
			ShowIP(client_addr);
			cout << endl;
			_resetIssued = true;  //Gets set back to false by the task that monitors it.
			close(tcp_connection);
			//Only allow it once every so often (the default RESET_INTERVAL was for 30 seconds - just
			//in case there is someone malicious on the network
			OSTimeDly(RESET_INTERVAL);
		}
	}
}

bool TcpServerReset::IsConnectionReset()
{
	return _resetIssued;
}

void TcpServerReset::ClearConnectionReset()
{
	_resetIssued = false;
}
